/////////////////////////////////////////
// main.cpp
//
// Applied Data Systems
// ------------------------------------------------------------------
// This is part of the COMTEST application
// 720020-1118x
// previously numbered as 700110-0011x and 20-1118x
//
// for more information see the header info in Serial.cpp
/////////////////////////////////////////

#include <windows.h>
#include <commctrl.h>

#include "resource.h"
#include "serial.h"
#include "Commdlg.h"

LRESULT			CALLBACK WndProc		(HWND, UINT, WPARAM, LPARAM);
BOOL			CALLBACK AboutDlgProc	(HWND, UINT, WPARAM, LPARAM) ;
HWND			DoCreateStatusBar		(HWND, int, HINSTANCE, int);

TCHAR		szAppName[]		= _T("ADS COMTEST Program");
TCHAR		szTitle[]		= _T("Serial Port Test");
HINSTANCE	hInst			= NULL;
HWND		hWndCB			= NULL;
HWND		hSendBox		= NULL;
HWND		hReceiveBox		= NULL;
HWND        hWnd			= NULL;
HWND		hWndSB			= NULL;
HANDLE		statusEvent		= NULL;
BOOL		sendFullBuffer	= FALSE;
BOOL		localEcho		= FALSE;
BOOL		captureText		= FALSE;
WNDPROC		hDefProc		= NULL;
char		sendBuffer[255];
FILE		*fCaptureFile	= NULL;

struct SBThreadInfo
{
	TCHAR	newText[255];
	int		duration;
	int		part;
};

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
                    LPTSTR lpCmdLine, int nCmdShow )
{
    MSG         msg;
    WNDCLASS    wc;
	DCB			dcb;
	int			err = 0;

	RETAILMSG(TRUE, (_T("---Starting COMTerm---\r\n")));

    wc.style          = NULL;
    wc.lpfnWndProc    = (WNDPROC) WndProc;
    wc.cbClsExtra     = 0;
    wc.cbWndExtra     = 0;
    wc.hInstance      = hInstance;
    wc.hIcon          = NULL;
    wc.hCursor        = NULL;
    wc.hbrBackground  = (HBRUSH) GetStockObject(WHITE_BRUSH);
    wc.lpszMenuName   = NULL;
    wc.lpszClassName  = szAppName;

    RegisterClass(&wc);

    InitCommonControls();   // Initialize common controls - command bar

    hInst = hInstance;      // Save handle to create command bar

	statusEvent = CreateEvent(NULL, TRUE, FALSE, _T("StatusChange"));

    hWnd = CreateWindow(szAppName,           // Class
                        szTitle,             // Title
                        WS_VISIBLE,          // Style
                        CW_USEDEFAULT,       // x-position
                        CW_USEDEFAULT,       // y-position
                        CW_USEDEFAULT,       // x-size
                        CW_USEDEFAULT,       // y-size
                        NULL,                // Parent handle
                        NULL,                // Menu handle
                        hInstance,           // Instance handle
                        NULL);               // Creation

    ShowWindow(hWnd, SW_SHOW);
    UpdateWindow(hWnd);

	// check available ports, open lowest available
	int lowport = CheckPorts();
	HMENU hMenu = CommandBar_GetMenu(hWndCB, 0);

	dcb.BaudRate = CBR_38400;
	dcb.Parity = NOPARITY;
	dcb.fRtsControl = FALSE;
	dcb.fDtrControl	= FALSE;
	dcb.StopBits = ONESTOPBIT;
    dcb.ByteSize = DATABITS_8;

	hCommPort = OpenCommsPort(hWnd, ports[lowport], dcb);
	StartReadThread();

	CheckMenuItem(hMenu, (ID_PORT_COM1 + lowport), MF_CHECKED);
	EnableMenuItem(hMenu, ID_PORT_COM1_OUT + lowport,  MF_GRAYED);

	while ( GetMessage(&msg, NULL, 0, 0) != FALSE ) 
	{
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}

	// close the communications port
	CloseCommsPort(hOutputPort);
	CloseCommsPort(hCommPort);

	// close the capture file if it's open
	if(captureText)
	{
		fclose(fCaptureFile);
	}

	RETAILMSG(TRUE, (_T("---Exiting COMTEST---\r\n")));

    return(msg.wParam);
}

HWND DoCreateStatusBar(HWND hwndParent, int nStatusID, HINSTANCE
                      hinst, int nParts)
{
    HWND hwndStatus;
    RECT rcClient;
    HLOCAL hloc;
    LPINT lpParts;
    int i, nWidth;

    // Ensure that the common control DLL is loaded.
    InitCommonControls();

    // Create the status bar.
    hwndStatus = CreateWindowEx(
        0,                       // no extended styles
        STATUSCLASSNAME,         // name of status bar class
        (LPCTSTR) NULL,          // no text when first created
        WS_CHILD | WS_VISIBLE,   // creates a child window
        0, 0, 0, 0,              // ignores size and position
        hwndParent,              // handle to parent window
        (HMENU) nStatusID,       // child window identifier
        hinst,                   // handle to application instance
        NULL);                   // no window creation data

    // Get the coordinates of the parent window's client area.
    GetClientRect(hwndParent, &rcClient);

    // Allocate an array for holding the right edge coordinates.
    hloc = LocalAlloc(LHND, sizeof(int) * nParts);
    lpParts = (int*)LocalLock(hloc);

    // Calculate the right edge coordinate for each part, and
    // copy the coordinates to the array.
    nWidth = rcClient.right / nParts;
    for (i = 0; i < nParts; i++) { 
       lpParts[i] = nWidth;
       nWidth += nWidth;
    }

    // Tell the status bar to create the window parts.
    SendMessage(hwndStatus, SB_SETPARTS, (WPARAM) nParts, (LPARAM)
               lpParts);

    // Free the array, and return.
    LocalUnlock(hloc);
    LocalFree(hloc);
    return hwndStatus;
}  

int CheckPorts()
{
	// check all available COM ports, return the lowest available
	BOOL	bFound	= FALSE;
	int		lowport = 0;
	HANDLE	hPort	= NULL;

	for(int i = 0 ; i < dim(ports) ; i++)
	{
		hPort = CreateFile (ports[i], GENERIC_READ | GENERIC_WRITE,
            0, NULL, OPEN_EXISTING, 0,NULL);

		if((hPort == 0) || ((DWORD)hPort == MAXDWORD))
		{
			HMENU hMenu = CommandBar_GetMenu(hWndCB, 0);

			EnableMenuItem(hMenu, (ID_PORT_COM1 + i), MF_GRAYED);
			EnableMenuItem(hMenu, (ID_PORT_COM1_OUT + i), MF_GRAYED);

			if(!bFound)
				lowport = i + 1;
		}
		else
		{
			// set it to the defaults or we kill the debug port
			DCB dcb;
			dcb.BaudRate = 38400;
			dcb.fParity = PARITY_NONE;
			dcb.StopBits = 1;
			dcb.ByteSize = 8;

			SetCommState(hPort, &dcb);

			bFound = TRUE;
			CloseHandle(hPort);
		}
	}

	return lowport;
}

VOID MenuHandler(HWND hWnd, INT idval)
{

	static UINT PortMenuItem	= ID_PORT_COM1;
	static UINT OutPortMenuItem = ID_SERIAL_RELAYPORT_NONE;
	static UINT SpeedMenuItem	= ID_SPEED_38400;
	static UINT FlowMenuItem	= ID_FLOWCONTROL_NONE;
	static UINT StopbitMenuItem  = ID_SERIAL_STOP_1BIT;
	static UINT ParityMenuItem  = ID_SERIAL_PARITY_NONE;
    static UINT DataBitsMenuItem = ID_DATA_8BITS;

	static BOOL InitializedDCB = FALSE;

	static DCB dcb;

	UINT newPortMenuItem = PortMenuItem;
	UINT newSpeedMenuItem = SpeedMenuItem;
	UINT newFlowMenuItem = FlowMenuItem;
	UINT newStopbitMenuItem = StopbitMenuItem;
	UINT newParityMenuItem = ParityMenuItem;
	UINT newOutPortMenuItem = OutPortMenuItem;
    UINT newDataBitsMenuItem = DataBitsMenuItem;
	UINT setEscape =0;

	HMENU hMenu = CommandBar_GetMenu(hWndCB, 0);
	
	if(InitializedDCB == FALSE)
	{
		dcb.BaudRate = CBR_38400;
		dcb.Parity = NOPARITY;
		dcb.fRtsControl = FALSE;
		dcb.fDtrControl	= FALSE;
		dcb.StopBits = ONESTOPBIT;
        dcb.ByteSize = 8;
		InitializedDCB = TRUE;
	}
	
	switch (idval)
	{	
		case ID_SERIAL_TEST_RECEIVEFILE:
			FileBufferLength = 0;
			FileReceive = TRUE;
			EnableMenuItem(hMenu, ID_SERIAL_TEST_RECEIVEFILE, MF_GRAYED); 
			EnableMenuItem(hMenu, ID_SERIAL_TEST_RETURNFILE,  MF_ENABLED); 
			return;
		case ID_SERIAL_TEST_RETURNFILE:
			FileReceive = FALSE;
			SendTextFile(hWnd);
			EnableMenuItem(hMenu, ID_SERIAL_TEST_RETURNFILE,  MF_GRAYED); 
			EnableMenuItem(hMenu, ID_SERIAL_TEST_RECEIVEFILE, MF_ENABLED); 
			return;

		case ID_SERIAL_TEST_SEND_A:
			SendTextN(hWnd,'A',200);
			return;
		case ID_SERIAL_TEST_SEND_B:
			SendTextN(hWnd,'B',200);
			return;
		case ID_SERIAL_TEST_SEND_DIGITS:
			SendTextDigits(hWnd,5);
			return;
		case ID_SERIAL_SENDA_STREAMOF_LOTSOF_DIGITS:
			SendTextDigits(hWnd,500);  // 500 lines x 64char/line ~ 32kB
			return;
		
	// the following all change the COM port settings so they drop through to 
	// close and reopen of the COM port and to check the appropriate menu items
		case ID_PORT_COM1:
			newPortMenuItem = ID_PORT_COM1;
			port_num = 1;
			break;
		case ID_PORT_COM2:
			newPortMenuItem = ID_PORT_COM2;
			port_num = 2;
			break;
		case ID_PORT_COM3:
			newPortMenuItem = ID_PORT_COM3;
			port_num = 3;
			break;
		case ID_PORT_COM4:
			newPortMenuItem = ID_PORT_COM4;
			port_num = 4;
			break;
		case ID_PORT_COM5:
			newPortMenuItem = ID_PORT_COM5;
			port_num = 5;
			break;
		case ID_PORT_COM6:
			newPortMenuItem = ID_PORT_COM6;
			port_num = 6;
			break;
		case ID_PORT_COM7:
			newPortMenuItem = ID_PORT_COM7;
			port_num = 7;
			break;
		case ID_PORT_COM8:
			newPortMenuItem = ID_PORT_COM8;
			port_num = 8;
			break;


		case ID_SERIAL_RELAYPORT_NONE:
			newOutPortMenuItem = ID_SERIAL_RELAYPORT_NONE;
			out_port_num = 0;
			break;
		case ID_PORT_COM1_OUT:
			newOutPortMenuItem = ID_PORT_COM1_OUT;
			out_port_num = 1;
			break;
		case ID_PORT_COM2_OUT:
			newOutPortMenuItem = ID_PORT_COM2_OUT;
			out_port_num = 2;
			break;
		case ID_PORT_COM3_OUT:
			newOutPortMenuItem = ID_PORT_COM3_OUT;
			out_port_num = 3;
			break;
		case ID_PORT_COM4_OUT:
			newOutPortMenuItem = ID_PORT_COM4_OUT;
			out_port_num = 4;
			break;
		case ID_PORT_COM5_OUT:
			newOutPortMenuItem = ID_PORT_COM5_OUT;
			out_port_num = 5;
			break;
		case ID_PORT_COM6_OUT:
			newOutPortMenuItem = ID_PORT_COM6_OUT;
			out_port_num = 6;
			break;
		case ID_PORT_COM7_OUT:
			newOutPortMenuItem = ID_PORT_COM7_OUT;
			out_port_num = 7;
			break;

		case ID_SPEED_1200:
			newSpeedMenuItem = idval;
			dcb.BaudRate = speed = CBR_1200;
			break;
		case ID_SPEED_2400:
			newSpeedMenuItem = idval;
			dcb.BaudRate = speed = CBR_2400;
			break;
		case ID_SPEED_4800:
			newSpeedMenuItem = idval;
			dcb.BaudRate = speed = CBR_4800;
			break;

		case ID_SPEED_9600:
			newSpeedMenuItem = idval;
			dcb.BaudRate = speed = CBR_9600;
			break;
		case ID_SPEED_19200:
			newSpeedMenuItem = idval;
			dcb.BaudRate = speed = CBR_19200;
			break;
		case ID_SPEED_38400:
			newSpeedMenuItem = idval;
			dcb.BaudRate = speed = CBR_38400;
			break;
		case ID_SPEED_57600:
			newSpeedMenuItem = idval;
			dcb.BaudRate = speed = CBR_57600;
			break;
		case ID_SPEED_115200:
			newSpeedMenuItem = idval;
			dcb.BaudRate = speed = CBR_115200;
			break;

		case ID_FLOWCONTROL_NONE:
			dcb.fRtsControl     = RTS_CONTROL_DISABLE;
			dcb.fDtrControl		= FALSE;
			dcb.fOutxCtsFlow	= FALSE;
			newFlowMenuItem = idval;
			break;

		case ID_FLOWCONTROL_XONXOFF:
			dcb.fOutX			= TRUE;
			dcb.fInX			= TRUE;
			newFlowMenuItem = idval;			
			break;
		
		case ID_FLOWCONTROL_CTSRTS:
			dcb.fOutxCtsFlow	= TRUE;
			dcb.fRtsControl		= RTS_CONTROL_HANDSHAKE;
			newFlowMenuItem = idval;
			break;

		case ID_FLOWCONTROL_RTSTOGGLE:
			dcb.fOutxCtsFlow	= TRUE;
			dcb.fRtsControl		= RTS_CONTROL_TOGGLE;
			newFlowMenuItem = idval;
			break;

		case ID_FLOWCONTROL_DSRDTR:

			dcb.fOutxDsrFlow	= TRUE;
			dcb.fDtrControl		= DTR_CONTROL_HANDSHAKE;
			newFlowMenuItem = idval;
			break;

		case ID_SERIAL_PARITY_NONE:
			newParityMenuItem = idval;
			dcb.Parity =NOPARITY;
			break;

		case ID_SERIAL_PARITY_EVEN:
			newParityMenuItem = idval;
			dcb.Parity =EVENPARITY;
			break;

		case ID_SERIAL_PARITY_ODD:

			newParityMenuItem = idval;
			dcb.Parity = ODDPARITY;
			break;

		case ID_SERIAL_STOP_1BIT:
			newStopbitMenuItem = idval;
			dcb.StopBits = ONESTOPBIT;
			break;
		case ID_SERIAL_STOP_15BIT:
			newStopbitMenuItem = idval;
			dcb.StopBits = ONE5STOPBITS;
			break;
		case ID_SERIAL_STOP_2BIT:
			newStopbitMenuItem = idval;
			dcb.StopBits = TWOSTOPBITS;
			break;

        case ID_DATA_8BITS:
            DEBUGMSG (1, (_T("ByteSize: 8 bits\r\n")));
            newDataBitsMenuItem = idval;
            dcb.ByteSize = 8;
            break;

        case ID_DATA_7BITS:
            DEBUGMSG (1, (_T("ByteSize: 7 bits\r\n")));
            newDataBitsMenuItem = idval;
            dcb.ByteSize = 7;
            dcb.fBinary = TRUE;
            dcb.fParity = TRUE;
            dcb.ErrorChar = 'X';
            dcb.EofChar = 0x0D;
            dcb.fNull = TRUE;
            break;

        case ID_DATA_6BITS:
            DEBUGMSG (1, (_T("ByteSize: 6 bits\r\n")));
            newDataBitsMenuItem = idval;
            dcb.ByteSize = 6;
            break;

        case ID_DATA_5BITS:
            DEBUGMSG (1, (_T("ByteSize: 5 bits\r\n")));
            newDataBitsMenuItem = idval;
            dcb.ByteSize = 5;
            break;

		case ID_ESC_SET_RTS:

		case ID_ESC_CLR_RTS:

		case ID_ESC_SET_DTR:

		case ID_ESC_CLR_DTR:

			setEscape = idval;
			break;

		case IDM_EXIT:
			DestroyWindow(hWnd);
			return;		
		case ID_TWIDDLE_DTR:
			Twiddle(0);
			return;
		case ID_TWIDDLE_RTS:
			Twiddle(1);
			return;
		case ID_SINGLE_BYTE:
			CheckMenuItem(hMenu, ID_SINGLE_BYTE, MF_CHECKED);
			CheckMenuItem(hMenu, ID_FULL_BUFFER, MF_UNCHECKED);
			sendFullBuffer = FALSE;
			return;
		case ID_FULL_BUFFER:
			CheckMenuItem(hMenu, ID_FULL_BUFFER, MF_CHECKED);
			CheckMenuItem(hMenu, ID_SINGLE_BYTE, MF_UNCHECKED);
			sendFullBuffer = TRUE;
			return;
		case ID_LOCALECHO_OFF:
			CheckMenuItem(hMenu, ID_LOCALECHO_OFF, MF_CHECKED);
			CheckMenuItem(hMenu, ID_LOCALECHO_ON, MF_UNCHECKED);
			localEcho = FALSE;
			return;
		case ID_LOCALECHO_ON:
			CheckMenuItem(hMenu, ID_LOCALECHO_ON, MF_CHECKED);
			CheckMenuItem(hMenu, ID_LOCALECHO_OFF, MF_UNCHECKED);
			localEcho = TRUE;
			return;
		
		case ID_CAPTURE:
			captureText = !captureText;
			if(captureText)
			{
				OPENFILENAME	ofn;
				TCHAR			fileName[MAX_PATH];
				TCHAR			title[MAX_PATH];
				TCHAR			ext[MAX_PATH];
				
				_tcscpy(title, _T("Capture Text To File"));
				_tcscpy(fileName, _T(""));
				_tcscpy(ext, _T("txt"));

				ofn.hwndOwner = hWnd;
				ofn.lpstrTitle = title;
				ofn.lpstrFile = fileName;
				ofn.nMaxFile = MAX_PATH;
				ofn.lpstrDefExt = ext;
				ofn.Flags = OFN_CREATEPROMPT | OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT;
				ofn.lStructSize = sizeof(OPENFILENAME);
				ofn.lpstrFilter = NULL;
				ofn.nFilterIndex = 0;
				ofn.lpstrFileTitle = NULL;
				ofn.lpstrInitialDir = NULL;

				if(GetSaveFileName(&ofn))
				{
					int len = _tcslen(ofn.lpstrFile) + 1;
					char *fname = (char*)malloc(len);
					len = WideCharToMultiByte(CP_OEMCP, 0, ofn.lpstrFile, len - 1, fname, len, NULL, NULL);
					
					fCaptureFile = fopen(fname, "wt");
					
					free(fname);

					if(fCaptureFile == NULL)
					{
						ChangeStatus(_T("Could not create file!"), INFINITE, 0);
					}
					else
					{
						CheckMenuItem(hMenu, ID_CAPTURE, MF_CHECKED);
						captureText = TRUE;
						ChangeStatus(_T("File Capture ON"), INFINITE, 2);
					}
				}
				break;
			} //if(captureText) 
			else
			{
				captureText = FALSE;
				CheckMenuItem(hMenu, ID_CAPTURE, MF_UNCHECKED);
				fclose(fCaptureFile);
				ChangeStatus(_T("File Capture OFF"), 5000, 2);
			}

			break;
		default:
			return;


	}

	CloseCommsPort(hCommPort);
	hCommPort = NULL;
	CloseCommsPort(hOutputPort);
	hOutputPort = NULL;

	Sleep(1);

	hCommPort = OpenCommsPort(hWnd, ports[port_num - 1], dcb);
	if(hCommPort)
		StartReadThread();

	if(out_port_num > 0)
	{
		hOutputPort = OpenCommsPort(hWnd, ports[out_port_num - 1], dcb);
	}

	CheckMenuItem(hMenu, PortMenuItem, MF_UNCHECKED); 
	CheckMenuItem(hMenu, OutPortMenuItem, MF_UNCHECKED); 
	CheckMenuItem(hMenu, SpeedMenuItem, MF_UNCHECKED); 
	CheckMenuItem(hMenu, FlowMenuItem, MF_UNCHECKED); 
	CheckMenuItem(hMenu, StopbitMenuItem, MF_UNCHECKED); 
	CheckMenuItem(hMenu, ParityMenuItem, MF_UNCHECKED); 
	CheckMenuItem(hMenu, DataBitsMenuItem, MF_UNCHECKED); 

	EnableMenuItem(hMenu, ID_PORT_COM1 + (OutPortMenuItem - ID_PORT_COM1_OUT),  MF_ENABLED);
	EnableMenuItem(hMenu, ID_PORT_COM1_OUT + (PortMenuItem - ID_PORT_COM1),  MF_ENABLED);

	PortMenuItem = newPortMenuItem;
	OutPortMenuItem = newOutPortMenuItem;
	SpeedMenuItem = newSpeedMenuItem;
	FlowMenuItem = newFlowMenuItem;
	StopbitMenuItem = newStopbitMenuItem;
	ParityMenuItem = newParityMenuItem;
    DataBitsMenuItem = newDataBitsMenuItem;

	if(out_port_num > 0)
	{
		EnableMenuItem(hMenu, ID_PORT_COM1 + (OutPortMenuItem - ID_PORT_COM1_OUT),  MF_GRAYED);
	}

	EnableMenuItem(hMenu, ID_PORT_COM1_OUT + (PortMenuItem - ID_PORT_COM1),  MF_GRAYED);

	CheckMenuItem(hMenu, PortMenuItem, MF_CHECKED); 
	CheckMenuItem(hMenu, OutPortMenuItem, MF_CHECKED); 
	CheckMenuItem(hMenu, SpeedMenuItem, MF_CHECKED); 
	CheckMenuItem(hMenu, FlowMenuItem, MF_CHECKED); 
	CheckMenuItem(hMenu, StopbitMenuItem, MF_CHECKED); 
	CheckMenuItem(hMenu, ParityMenuItem, MF_CHECKED); 
	CheckMenuItem(hMenu, DataBitsMenuItem, MF_CHECKED); 

	// RTS
	if (RtsControl==RTS_CONTROL_ENABLE)
		CheckMenuItem(hMenu, ID_TOGGLE_RTS, MF_CHECKED);
	else
		CheckMenuItem(hMenu, ID_TOGGLE_RTS, MF_UNCHECKED);

	// DTR
	if (DtrControl==DTR_CONTROL_ENABLE)
		CheckMenuItem(hMenu, ID_TOGGLE_DTR, MF_CHECKED);
	else
		CheckMenuItem(hMenu, ID_TOGGLE_DTR, MF_UNCHECKED);

	if(setEscape != 0)
	{
		SetEscapeFunc(setEscape);
	}
}

LRESULT CALLBACK TextWndProc(HWND hWnd, UINT message,
                          WPARAM wParam, LPARAM lParam )
{
	char	c[3];
	int		textLen;

    switch (message)
    {
		case WM_CHAR:
			c[0] = (char)wParam;
			c[1] = '\0';

			if(c[0] == 13)
			{
				// scroll to the bottom of the text box
				textLen = GetWindowTextLength(hReceiveBox);
				SendMessage(hReceiveBox, EM_SETSEL, textLen, textLen);
				SendMessage(hReceiveBox, EM_SCROLLCARET, textLen, textLen);

				strcat(sendBuffer, "\r");
				SendText(sendBuffer);
				sendBuffer[0] = '\0';
			}
			else
			{
				if(sendFullBuffer)
				{
					strcat(sendBuffer, c);
				}
				else
				{
					SendText(c);
				}
			}

			if(! localEcho)
				break;

			if(captureText)
			{
				fwrite(&c[0], sizeof(char), strlen(c), fCaptureFile);
			}
		default:
			return CallWindowProcW(hDefProc, hWnd, message, wParam, lParam);
    }
	
    return (0);
}

LRESULT CALLBACK WndProc(HWND hWnd, UINT message,
                          WPARAM wParam, LPARAM lParam )
{
    HDC          hdc;
    PAINTSTRUCT  ps;
	BOOL b;

    switch (message)
    {
	case WM_SETFOCUS:
		SetFocus(hSendBox);
		break;

	case WM_COMMAND:
		switch(LOWORD(wParam))
		{
			case IDM_APP_ABOUT:
				DialogBox(hInst,MAKEINTRESOURCE(IDD_DIALOG1),hWnd,AboutDlgProc);
				break;

		}

		MenuHandler(hWnd, LOWORD(wParam));

		break;

    case WM_CREATE:
		
        hWndCB = CommandBar_Create(hInst, hWnd, 1);
		
		if(!CommandBar_InsertMenubar(hWndCB, hInst, IDR_MAINMENU, 0))
		{
			b = GetLastError();
		}

        CommandBar_AddAdornments(hWndCB, 0, NULL);

		hReceiveBox = CreateWindow(
					TEXT("Edit"),		// class name
					NULL,				// caption
					WS_HSCROLL | ES_MULTILINE | WS_CHILD | WS_VISIBLE | WS_BORDER | WS_VSCROLL ,	// style
					0, 0, 0, 0,			// don't care about the position yet
					hWnd,				// parent
					(HMENU) 101,		// window ID
					hInst,				// instance handle of application
					NULL				// no information to pass on
					);
		SetFocus(hReceiveBox);
		hDefProc = (WNDPROC)SetWindowLong(hReceiveBox, GWL_WNDPROC, (long)TextWndProc);

		SendMessage(hReceiveBox, EM_LIMITTEXT, -1, 0);

		// init the send buffer
		strcpy(sendBuffer, "");

		hWndSB = DoCreateStatusBar(hWnd, 1001, hInst, 3);		

        break;

    case WM_PAINT:
        hdc = BeginPaint(hWnd, &ps);
        EndPaint(hWnd, &ps);
        break;

    case WM_HIBERNATE:
        MessageBox(NULL, TEXT("MEMORY IS LOW"), szAppName, MB_OK);
		break;

    case WM_CLOSE:
        DestroyWindow(hWnd);
        break;

    case WM_DESTROY:
        PostQuitMessage(0);
        break;

	case WM_SIZE:			
		// Application window has been resized, ensure that two
		// edit boxes are correctly positioned.
		int nCmdHeight;
		RECT rct;

		nCmdHeight = CommandBar_Height(hWndCB);
		GetWindowRect(hWndCB, &rct);

		MoveWindow(hReceiveBox, 
						0,							// x
						nCmdHeight,	// y
						LOWORD(lParam),				// width
						HIWORD(lParam) - nCmdHeight - (rct.bottom - rct.top) + 5,  // height
						TRUE);
		break;
    default:
        return (DefWindowProc(hWnd, message, wParam, lParam));
    }
    return (0);
}

BOOL CALLBACK AboutDlgProc (HWND hDlg, UINT message, 
                            WPARAM wParam, LPARAM lParam)
{     
	switch (message)
    {     
	case WM_INITDIALOG :          
		return TRUE ;          
    case WM_COMMAND :          
		switch (LOWORD (wParam))          
		{
			case IDOK :          
			case IDCANCEL :
				EndDialog (hDlg, 0) ;               
				return TRUE ;          
		}
        break ;     
	}     
	return FALSE ;
}

void ChangeStatus(TCHAR *statusText, int displayDuration, int part)
{
	DWORD			SBThread;
	SBThreadInfo	info;

	if(displayDuration > 0)
	{
		// create thread parameter values
		info.duration = displayDuration;
		info.part = part;
		_tcscpy((TCHAR*)info.newText, statusText);
	
		// reset the event
		ResetEvent(statusEvent);

		// start the thread
		CreateThread(NULL, 0, StatusThreadProc, &info, NULL, &SBThread);

		// wait for the thread to extract the params (max 20 sec)
		WaitForSingleObject(statusEvent, 20000);

		// reset the event
		ResetEvent(statusEvent);
	}
	else
	{
		SendMessage(hWndSB, SB_SETTEXT, part, (LPARAM)statusText);
	}
}

DWORD WINAPI StatusThreadProc(void *lpVoid)
{
	// extract thread parameter values
	SBThreadInfo info;
	memcpy(&info, lpVoid, sizeof(SBThreadInfo));
	
	// let caller know we've gotten values
	SetEvent(statusEvent);

	// set the next text
	SendMessage(hWndSB, SB_SETTEXT, info.part, (LPARAM)info.newText);

	// wait
	Sleep(info.duration);

	SendMessage(hWndSB, SB_SETTEXT, info.part, (LPARAM)_T(""));

	return 0;
}